#!/tvbin/tivosh
# ************************************************************
# AltEPG Project Source Code Version Control
# $FileID: git::root/hack/bin/setNextCall.tcl $
# $Revision: altepg1.0c root 2011-09-17 23:39:32 +0100 Finalising altepg1.0c $
# Description: Adjust next call time after tclient finishes
# ************************************************************

## setNextCall earliestPermittedTime [delayseconds] ["test"]

# Waits 'delayseconds' (default = 0) then checks the next scheduled call time. If it before the specified time, adds whole hours
# to the time until it is after that time.

# If the time difference between the revised next scheduled time and the last successful call is more than 36 hours, invokes a new (semi-automatic)
# call as soon as possible

# if param 3 is "test" then it just reports on what it would do rather than doing it

# (potential bug: this script is typically invoked from TClient, which starts it as a background task. This script then seems
#  to inherit the setlog setting from TClient, so putlog logs to /var/log/tclient.  However if TClient terminates
#  successfully *before* this script finishes, then tcphonehome rotates the logs, and subsequent putlogs from this script
#  are lost - or they might even cause the script to fail?)

# Magic numbers

set maxCallIntervalHours 36

# Physical constants

set secsPerHour 3600

source $tcl_library/tv/Inc.itcl
setlog /var/log/setnextcall 1

proc stringTime { timeval } {
  return  [clock format $timeval -format %Y/%m/%d:%H:%M:%S]
}

proc sleep { seconds } {
  after [expr $seconds * 1000]
}

# collect command line parameters

set earliestPermittedTime [lindex $argv 0]
set delay [lindex $argv 1]
set mode [lindex $argv 2]

if { $delay == "" } {
  set delay 0
}

set earliestPermittedTimeStr [stringTime $earliestPermittedTime]
putlog "setting nextcall to be no sooner than $earliestPermittedTime ($earliestPermittedTimeStr) after a delay of $delay seconds"

if { $delay != 0 } {
  putlog "sleeping $delay secs"
  if { $mode == "test" } {
    putlog "TEST: sleep $delay"
  } else {
    sleep $delay
  }
  putlog "awake again"
}

set db [dbopen]

try {
  # delay the next call if needed
  
  RetryTransaction {
    set sobj [db $db open /Setup]
    set lastSuccess [dbobj $sobj get LastSuccessCall]
    set currentNextCall  [dbobj $sobj get NextCallAttempt]
    set currentNextCallStr [stringTime $currentNextCall]
    set now [clock seconds]
    if { $currentNextCall < $earliestPermittedTime } {
      set nextCall $currentNextCall
      while { $nextCall < $earliestPermittedTime } {
	  set nextCall [expr $nextCall + $secsPerHour]
      }
      if { $mode == "test" } {
	putlog "TEST: dbobj $sobj set NextCallAttempt $nextCall"
      } else {
        dbobj $sobj set NextCallAttempt $nextCall
      }
      set nextCallStr [stringTime $nextCall]
      putlog "moved NextCallAttempt back from $currentNextCall ($currentNextCallStr) to $nextCall ($nextCallStr)"
      set currentNextCall $nextCall
    } else {
      putlog "NextCallAttempt $currentNextCall ($currentNextCallStr) is after $earliestPermittedTime ($earliestPermittedTimeStr) so OK"
    }
  }

  # check whether we need to schedule an extra call
  
  set maxCallInterval [expr $maxCallIntervalHours * $secsPerHour ]
  set lastSuccessStr [stringTime $lastSuccess]
  putlog "last successful call was at $lastSuccess ($lastSuccessStr)"
  if { [expr $currentNextCall - $lastSuccess] > $maxCallInterval } {
    set now [clock seconds]
    while { $now < $earliestPermittedTime } {
      putlog "Sleeping until $earliestPermittedTime ($earliestPermittedTimeStr) to schedule extra call"
      if { $mode == "test" } {
        putlog "TEST: sleep $secsPerHour"
	set now $earliestPermittedTime
      } else {
	sleep $secsPerHour
      }
      set now [clock seconds]
    }
    set nowStr [stringTime $now]
    putlog "Time now $now ($nowStr)"
    putlog "Sending event to perform daily call NOW - time should after $earliestPermittedTime ($earliestPermittedTimeStr)"
    if { $mode == "test" } {
      putlog "TEST: event send $TmkEvent::EVT_DIALUPEVENT 4"
    } else {
      set fid [open /var/tmp/tclient-automatic w 0755]
      puts $fid "fake automatic tclient call"
      close $fid
      event send $TmkEvent::EVT_DIALUPEVENT 4    
      set dialupReq [binary format c20 "127 -60 126 100 127 -1 -15 -80 1 -78 92 -56 127 -18 -5 116 127 -75 -91 -8"]
#      event send $TmkEvent::EVT_DIALUPEVENT 0 $dialupReq
    }
  }
} catch errCode {
  putlog "error in setNextCall.tcl: $errCode"
}
